<?php

/**
 * @author        Rein de Vries <support@reinos.nl>
 * @link        http://addons.reinos.nl
 * @copyright    Copyright (c) 2011 - 2021 Reinos.nl Internet Media
 * @license     http://addons.reinos.nl/commercial-license
 *
 * Copyright (c) 2011 - 2021 Reinos.nl Internet Media
 * All rights reserved.
 *
 * This source is commercial software. Use of this software requires a
 * site license for each domain it is used on. Use of this software or any
 * of its source code without express written permission in the form of
 * a purchased commercial or other license is prohibited.
 *
 * THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
 * KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
 * PARTICULAR PURPOSE.
 *
 * As part of the license agreement for this software, all modifications
 * to this source must be submitted to the original author for review and
 * possible inclusion in future releases. No compensation will be provided
 * for patches, although where possible we will attribute each contribution
 * in file revision notes. Submitting such modifications constitutes
 * assignment of copyright to the original author (Rein de Vries and
 * Reinos.nl Internet Media) for such modifications. If you do not wish to assign
 * copyright to the original author, your license to  use and modify this
 * source is null and void. Use of this software constitutes your agreement
 * to this clause.
 */

namespace Reinos\Mollie\Service;

require_once PATH_THIRD . 'reinos_mollie/config.php';

class Checkout
{

    public $requiredFields = array(
        'billing_address' => true,
        'billing_address2' => false,
        'billing_city' => true,
        'billing_country' => true,
        'billing_zip' => true,
        'shipping_address' => true,
        'shipping_address2' => false,
        'shipping_city' => true,
        'shipping_country' => true,
        'shipping_zip' => true,
        'customer_first_name' => true,
        'customer_last_name' => true,
        'customer_email' => true,
        'customer_phone' => false,
        'customer_company' => false,
    );

    public $orderData = array();

    public $errors = array();

    //---------------------------------------------------------------------------------------------------

    public function __construct()
    {
        if (session_id() == "") {
            session_start();
        }
    }

    public function init()
    {
        //save the data to the $orderData
        $this->saveData();

        //check if the country is selected, or a default one is selected
        $this->setDefaultCountry();

        //check if the billing is the same as shipping
        $this->checkShippingIsSameAsBilling();

        //check if the needed values are filled in
        $this->checkRequired();

        //validate email
        $this->validateEmail();

        //check if the needed values are filled in
        $this->checkItems();

        //save the data to the session
        $this->saveToSession();

        //check if we have any errors?
        if (!empty($this->errors)) {
            ee()->session->set_flashdata(REINOS_MOLLIE_SERVICE_NAME . ':errors', $this->errors);
            ee()->load->helper('url_helper');
            redirect(ee()->input->post('current_url'));
            exit;
        }

        //get the cart items
        $items = ee(REINOS_MOLLIE_SERVICE_NAME . ':Cart')->getItems();

        //set the data
        $this->orderData = array_merge($this->orderData, $items);

        //create a new model record
        $order = ee('Model')->make(REINOS_MOLLIE_MAP . ':Order')->set($this->orderData);

        //set the return url and current URL
        $order->return_url = ee()->input->post('return_url');
        $order->checkout_url = ee()->input->post('current_url');

        //set the current time
        $order->time = ee()->localize->now;

        //default status
        $order->status = 'open';

        //set the hash
        $order->hash = uniqid();

        //set the site_id
        $order->site_id = ee()->config->item('site_id');

        //save the data, to obtain a new entry_id
        $order->save();

        //Check if we handle it with Mollie or just manual
        if (ee(REINOS_MOLLIE_SERVICE_NAME . ':Settings')->item('mollie_mode') != 'none') {
            //Init Mollie
            ee(REINOS_MOLLIE_SERVICE_NAME . ':Payment')->init();

            //create payment
            $payment = ee(REINOS_MOLLIE_SERVICE_NAME . ':Payment')->create($order->total_plus_tax, $order->order_id,
                $order->hash);

            //After creation, the payment id is available in the $payment->id property. You should store this id with your order.
            $order->transaction_id = $payment->id;
            $order->payment_url = $payment->getCheckoutUrl();
            $order->save();

            //After storing the payment id you can send the customer to the checkout using the $payment->getCheckoutUrl().
            header("Location: " . $payment->getCheckoutUrl(), true, 303);
        } else {
            //set the manual status
            $order->status = 'manual';
            $order->save();

            header("Location: " . ee(REINOS_MOLLIE_SERVICE_NAME . ':Settings')->act_url . '&method=redirect&hash=' . $order->hash,
                true, 303);
        }
    }

    //---------------------------------------------------------------------------------------------------

    public function getFieldValues()
    {
        $return = array();
        $values = isset($_SESSION[REINOS_MOLLIE_MAP]['orderData']) ? $_SESSION[REINOS_MOLLIE_MAP]['orderData'] : array();
        $fields = array_keys($this->requiredFields);

        if (!empty($fields)) {
            foreach ($fields as $field) {
                if (isset($values[$field])) {
                    $return[$field . ':value'] = $values[$field];
                } else {
                    $return[$field . ':value'] = '';
                }
            }
        }

        return $return;
    }

    //---------------------------------------------------------------------------------------------------

    private function checkItems()
    {
        if (ee(REINOS_MOLLIE_SERVICE_NAME . ':Cart')->count() == 0) {
            $this->errors['no_items_to_checkout'] = ee()->lang->line(REINOS_MOLLIE_MAP . '_no_items_to_checkout');
        }
    }

    //---------------------------------------------------------------------------------------------------

    private function validateEmail()
    {
        if (!filter_var($this->orderData['customer_email'], FILTER_VALIDATE_EMAIL)) {
            if (!isset($this->errors['customer_email'])) {
                $this->errors['customer_email'] = array();
            }
            $this->errors['customer_email'] = ee()->lang->line(REINOS_MOLLIE_MAP . '_invalid_email');
        }
    }

    //---------------------------------------------------------------------------------------------------

    private function checkRequired()
    {
        foreach ($this->orderData as $key => $value) {
            if (isset($this->requiredFields[$key]) && $this->requiredFields[$key] === true) {
                if ($value == '') {
                    if (!isset($this->errors[$key])) {
                        $this->errors[$key] = array();
                    }
                    $this->errors[$key] = ee()->lang->line(REINOS_MOLLIE_MAP . '_required');
                }
            }
        }
    }

    //---------------------------------------------------------------------------------------------------

    private function saveData()
    {
        $this->orderData['billing_address'] = ee()->input->post('billing_address');
        $this->orderData['billing_address2'] = ee()->input->post('billing_address2');
        $this->orderData['billing_zip'] = ee()->input->post('billing_zip');
        $this->orderData['billing_city'] = ee()->input->post('billing_city');
        $this->orderData['billing_country'] = ee()->input->post('billing_country');

        $this->orderData['shipping_address'] = ee()->input->post('shipping_address');
        $this->orderData['shipping_address2'] = ee()->input->post('shipping_address2');
        $this->orderData['shipping_zip'] = ee()->input->post('shipping_zip');
        $this->orderData['shipping_city'] = ee()->input->post('shipping_city');
        $this->orderData['shipping_country'] = ee()->input->post('shipping_country');

        $this->orderData['customer_first_name'] = ee()->input->post('customer_first_name');
        $this->orderData['customer_last_name'] = ee()->input->post('customer_last_name');
        $this->orderData['customer_email'] = ee()->input->post('customer_email');
        $this->orderData['customer_company'] = ee()->input->post('customer_company');
        $this->orderData['customer_phone'] = ee()->input->post('customer_phone');
    }

    //---------------------------------------------------------------------------------------------------

    private function checkShippingIsSameAsBilling()
    {
        if (isset($_POST['billing_same_as_shipping']) && ee()->input->post('billing_same_as_shipping') == 1) {
            $this->orderData['shipping_address'] = $this->orderData['billing_address'];
            $this->orderData['shipping_zip'] = $this->orderData['billing_zip'];
            $this->orderData['shipping_city'] = $this->orderData['billing_city'];
            $this->orderData['shipping_country'] = $this->orderData['billing_country'];
        }
    }

    //---------------------------------------------------------------------------------------------------

    private function setDefaultCountry()
    {
        if ($this->orderData['billing_country'] == '') {
            $this->orderData['billing_country'] = ee(REINOS_MOLLIE_SERVICE_NAME . ':Settings')->item('default_country');
        }

        if ($this->orderData['shipping_country'] == '') {
            $this->orderData['shipping_country'] = ee(REINOS_MOLLIE_SERVICE_NAME . ':Settings')->item('shipping_country');
        }
    }

    private function saveToSession()
    {
        $_SESSION[REINOS_MOLLIE_MAP]['orderData'] = $this->orderData;
    }

    private function currentUrl()
    {
        $protocol = ((!empty($_SERVER['HTTPS']) && $_SERVER['HTTPS'] != 'off') || $_SERVER['SERVER_PORT'] == 443) ? "https://" : "http://";
        return $protocol . $_SERVER['HTTP_HOST'] . $_SERVER['REQUEST_URI'] . (isset($_POST) ? '?' . http_build_query($_POST) : '') . (isset($_GET) ? '?' . http_build_query($_GET) : '');
    }
}
